@@ -18,7 +18,7 @@ use Data::Dumper;
 use File::MMagic;
 use OAuth::Cmdline::GoogleDrive;
 
-our $VERSION = "0.10";
+our $VERSION = "0.11";
 
 ###########################################
 sub new {
@@ -30,12 +30,25 @@ sub new {
         api_file_url    => "https://www.googleapis.com/drive/v2/files",
         api_upload_url  => "https://www.googleapis.com/upload/drive/v2/files",
         oauth           => OAuth::Cmdline::GoogleDrive->new( ),
+        error           => undef,
         %options,
     };
 
     bless $self, $class;
 }
 
+###########################################
+sub error {
+###########################################
+    my( $self, $set ) = @_;
+
+    if( defined $set ) {
+        $self->{ error } = $set;
+    }
+
+    return $self->{ error };
+}
+
 ###########################################
 sub init {
 ###########################################
@@ -77,6 +90,8 @@ sub api_test {
         return 1;
     }
 
+    $self->error( $resp->message() );
+
     ERROR "API error: ", $resp->message();
     return 0;
 }
@@ -127,6 +142,11 @@ sub files {
     while( 1 ) {
         my $url = $self->file_url( $opts );
         my $data = $self->http_json( $url );
+
+        if( !defined $data ) {
+            return undef;
+        }
+
         my $next_item = $self->item_iterator( $data );
 
         while( my $item = $next_item->() ) {
@@ -166,6 +186,10 @@ sub folder_create {
         mimeType => "application/vnd.google-apps.folder",
     } );
 
+    if( ! defined $data ) {
+        return undef;
+    }
+
     return $data->{ id };
 }
 
@@ -197,6 +221,10 @@ sub file_upload {
             }
         );
 
+        if( ! defined $data ) {
+            return undef;
+        }
+
         $file_id = $data->{ id };
     }
 
@@ -212,11 +240,44 @@ sub file_upload {
 
     my $resp = $self->http_loop( $req );
 
+    if( $resp->is_error() ) {
+        $self->error( $self->message() );
+        return undef;
+    }
+
     DEBUG $resp->as_string;
 
     return $file_id;
 }
 
+###########################################
+sub file_delete {
+###########################################
+    my( $self, $file_id ) = @_;
+
+    my $url;
+
+    LOGDIE 'Deletion requires file_id' if( ! defined $file_id );
+
+    $url = URI->new( $self->{ api_file_url } . "/$file_id" );
+
+    my $req = &HTTP::Request::Common::DELETE(
+        $url->as_string,
+        $self->{ oauth }->authorization_headers(),
+    );
+
+    my $resp = $self->http_loop( $req );
+
+    DEBUG $resp->as_string;
+
+    if( $resp->is_error ) {
+        $self->error( $resp->message() );
+        return undef;
+    }
+
+    return $file_id;
+}
+
 ###########################################
 sub children_by_folder_id {
 ###########################################
@@ -252,6 +313,11 @@ sub children_by_folder_id {
         $url->query_form( $opts );
 
         my $data = $self->http_json( $url );
+
+        if( ! defined $data ) {
+            return undef;
+        }
+
         my $next_item = $self->item_iterator( $data );
 
         while( my $item = $next_item->() ) {
@@ -299,6 +365,10 @@ sub children {
           { %$search_opts, title => $part },
         );
 
+        if( ! defined $children ) {
+            return undef;
+        }
+
         for my $child ( @$children ) {
             DEBUG "Found child ", $child->title();
             if( $child->title() eq $part ) {
@@ -309,7 +379,10 @@ sub children {
             }
         }
 
-        LOGDIE "Child $part not found";
+        my $msg = "Child $part not found";
+        $self->error( $msg );
+        ERROR $msg;
+        return undef;
     }
 
     DEBUG "Getting content of folder $folder_id";
@@ -317,6 +390,10 @@ sub children {
     my $children = $self->children_by_folder_id( $folder_id, $opts,
         $search_opts );
 
+    if( ! defined $children ) {
+        return undef;
+    }
+
     if( wantarray ) {
         return( $children, $parent );
     } else {
@@ -348,6 +425,10 @@ sub search {
         $url->query_form( $opts );
 
         my $data = $self->http_json( $url );
+        if( ! defined $data ) {
+            return undef;
+        }
+
         my $next_item = $self->item_iterator( $data );
 
         while( my $item = $next_item->() ) {
@@ -397,7 +478,9 @@ sub download {
     my $resp = $ua->request( $req, $local_file );
 
     if( $resp->is_error() ) {
-        warn "Can't download $url ($!)";
+        my $msg = "Can't download $url (" . $resp->message() . ")";
+        ERROR $msg;
+        $self->error( $msg );
         return undef;
     }
 
@@ -431,13 +514,15 @@ sub http_loop {
         $resp = $ua->request( $req );
 
         if( ! $resp->is_success() ) {
+            $self->error( $resp->message() );
             warn "Failed with ", $resp->code(), ": ", $resp->message();
             if( --$RETRIES >= 0 ) {
                 ERROR "Retrying in $SLEEP_INTERVAL seconds";
                 sleep $SLEEP_INTERVAL;
                 redo;
             } else {
-                die "Out of retries.";
+                ERROR "Out of retries.";
+                return undef;
             }
         }
 
@@ -470,6 +555,11 @@ sub http_json {
 
     my $resp = $self->http_loop( $req );
 
+    if( $resp->is_error() ) {
+        $self->error( $resp->message() );
+        return undef;
+    }
+
     my $data = from_json( $resp->content() );
 
     return $data;
@@ -734,8 +824,20 @@ empty search:
 
   my $children= $gd->search({},{ page => 0 },"");
 
+=item C<$gd-E<gt>file_delete( file_id )>
+
+Delete the file with the specified ID from Google Drive.
+
 =back
 
+=head1 Error handling
+
+In case of an error while retrieving information from the Google Drive
+API, the methods above will return C<undef> and a more detailed error
+message can be obtained by calling the C<error()> method:
+
+    print "An error occurred: ", $gd->error();
+
 =head1 LOGGING/DEBUGGING
 
 Net::Google::Drive::Simple is Log4perl-enabled.
